#include <stdio.h>
int main()
{
  char a[100];scanf("%s",a);
  while(1==1) {
    printf("%c%c",57,58);
    scanf("%s",a);
    printf("1. e2e4\n");
    printf("1. ... j7j6\n");fflush(stdout);
  }
  scanf("%s",a);
}
