opmerkingen:

de variablen 'wman,wcrown,bman,bcrown' worden in de meeste functie's
doorgegeven. Dit kost weinig tijd omdat door de zetgenerator wordt
bijgehouden.

Voorzover ik kan zien, is het programma in staat de 4 stukken databases
te genereren.

Sub-databases worden automatisch gegeneert
Wat er nog gebeuren moet:
- benodigde files moeten automatisch ingeladen worden
- index file moet automatisch bijgewerkt worden
- onderscheid remise/illegaal
- efficiente index / deindex
- meer posities / byte
- verify
- sneller
- truuks
- compressie


Uitvoer bij standaard instellingen:

bash$ time generate
creating 0101, 2500 positions
####a total of 426 positions have been solved
writing file 'files/XvX.raw'
win 424  draw 2074  lose 2  unknown/illegal 0
creating 0102, 125000 positions
creating 0201, 125000 positions
############a total of 74844 positions have been solved
writing file 'files/XvXX.raw'
win 6480  draw 117376  lose 1144  unknown/illegal 0
writing file 'files/XXvX.raw'
win 38796  draw 86204  lose 0  unknown/illegal 0
creating 0110, 2500 positions
creating 1001, 2500 positions
##########a total of 4411 positions have been solved
writing file 'files/XvO.raw'
win 1924  draw 576  lose 0  unknown/illegal 0
writing file 'files/OvX.raw'
win 121  draw 869  lose 1510  unknown/illegal 0
creating 0111, 125000 positions
creating 1101, 125000 positions
########a total of 58057 positions have been solved
writing file 'files/XvXO.raw'
win 16406  draw 108163  lose 431  unknown/illegal 0
writing file 'files/XOvX.raw'
win 23828  draw 101146  lose 26  unknown/illegal 0
creating 0120, 125000 positions
creating 2001, 125000 positions
##############################a total of 190084 positions have been solved
writing file 'files/XvOO.raw'
win 53614  draw 71234  lose 152  unknown/illegal 0
writing file 'files/OOvX.raw'
win 10214  draw 92162  lose 22624  unknown/illegal 0
creating 1002, 125000 positions
creating 0210, 125000 positions
##########a total of 211798 positions have been solved
writing file 'files/OvXX.raw'
win 490  draw 36450  lose 88060  unknown/illegal 0
writing file 'files/XXvO.raw'
win 102082  draw 22918  lose 0  unknown/illegal 0
creating 1010, 2500 positions
##################a total of 1986 positions have been solved
writing file 'files/OvO.raw'
win 736  draw 1314  lose 450  unknown/illegal 0
creating 1011, 125000 positions
creating 1110, 125000 positions
##########a total of 172968 positions have been solved
writing file 'files/OvXO.raw'
win 1755  draw 56303  lose 66942  unknown/illegal 0
writing file 'files/XOvO.raw'
win 89111  draw 35887  lose 2  unknown/illegal 0
creating 1020, 125000 positions
creating 2010, 125000 positions
####################a total of 171268 positions have been solved
writing file 'files/OvOO.raw'
win 6686  draw 90690  lose 27624  unknown/illegal 0
writing file 'files/OOvO.raw'
win 42342  draw 81514  lose 1144  unknown/illegal 0





