/*
 * bitmaps.h - Include bitmap files for pieces and icons
 * $Id: bitmaps.h,v 1.2 1995/03/09 00:43:28 mann Exp $
 *
 * Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts.
 * Enhancements Copyright 1992-95 Free Software Foundation, Inc.
 *
 * The following terms apply to Digital Equipment Corporation's copyright
 * interest in XBoard:
 * ------------------------------------------------------------------------
 * All Rights Reserved
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of Digital not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 * ------------------------------------------------------------------------
 *
 * The following terms apply to the enhanced version of XBoard distributed
 * by the Free Software Foundation:
 * ------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ------------------------------------------------------------------------
 *
 * See the file ChangeLog for a revision history.
 */

#include "bitmaps/p80s.bm"
#include "bitmaps/n80s.bm"
#include "bitmaps/b80s.bm"
#include "bitmaps/r80s.bm"
#include "bitmaps/q80s.bm"
#include "bitmaps/k80s.bm"

#include "bitmaps/p80o.bm"
#include "bitmaps/n80o.bm"
#include "bitmaps/b80o.bm"
#include "bitmaps/r80o.bm"
#include "bitmaps/q80o.bm"
#include "bitmaps/k80o.bm"

#include "bitmaps/p64s.bm"
#include "bitmaps/n64s.bm"
#include "bitmaps/b64s.bm"
#include "bitmaps/r64s.bm"
#include "bitmaps/q64s.bm"
#include "bitmaps/k64s.bm"

#include "bitmaps/p64o.bm"
#include "bitmaps/n64o.bm"
#include "bitmaps/b64o.bm"
#include "bitmaps/r64o.bm"
#include "bitmaps/q64o.bm"
#include "bitmaps/k64o.bm"

#include "bitmaps/p40s.bm"
#include "bitmaps/n40s.bm"
#include "bitmaps/b40s.bm"
#include "bitmaps/r40s.bm"
#include "bitmaps/q40s.bm"
#include "bitmaps/k40s.bm"

#include "bitmaps/p40o.bm"
#include "bitmaps/n40o.bm"
#include "bitmaps/b40o.bm"
#include "bitmaps/r40o.bm"
#include "bitmaps/q40o.bm"
#include "bitmaps/k40o.bm"

#include "bitmaps/icon_white.bm"
#include "bitmaps/icon_black.bm"
#include "bitmaps/checkmark.bm"
